#
# This file is part of pysimpledb.
#
# pysimpledb is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pysimpledb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pysimpledb.  If not, see <http://www.gnu.org/licenses/>.
#
#      Author Daniel Mikusa <dan@mikusa.com>
#   Copyright Daniel Mikusa 2008
#
from pysimpledb.sql import Sql
from pysimpledb.mappers import *

class Tea:
    """Simple data object, no parameters are needed"""
    def __init__(self, id = None, name = None, cost = 0.0):
        self.id = id
        self.name = name
        self.cost = cost
        self.mod = datetime.now()

    def __str__(self):
        return ("Tea : [id -> %s, name -> %s, cost -> %s, mod -> %s]" %
                                    (self.id, self.name, self.cost, self.mod))

    def __repr__(self):
        return str(self)

class TeaSql(Sql):
    """Dao Object the long way (does not generate dynamic functions)"""
    def __init__(self, db):
        """
        Define queries and database connection then passes them to super class.

        First define all of your queries in a dictionary.  The key will need
        to be passed to the Dao when an operation is executed.  The value is
        another dictionary which defines the query.

        For details on the different options see the library documentation.

        The following example attempts to define as many of the different
        options as possible.

        Finally when all of your queries have been defined, simply pass the
        dictionary of queries and the database connection to the super class.
        """
        queries = {
            'get': {
                'sql': 'SELECT * FROM tea WHERE id = :id'
            },
            'getAll': {
                'sql': 'SELECT * FROM tea',
                'rowMapper': ObjectRowMapper(Tea)
            },
            'getTeaByName': {
                'sql': 'SELECT * FROM tea WHERE name = :name'
            },
            'count': {
                'sql': 'SELECT COUNT(*) FROM tea'
            },
            'sum': {
                'sql': 'SELECT SUM(cost) FROM tea',
                'rowMapper': SingleTypeRowMapper(float)
            },
            'avg': {
                'sql': 'SELECT AVG(cost) FROM tea'
            },
            'insert': {
                'sql': 'INSERT INTO tea (name, cost, mod) VALUES (:name, :cost, :mod)',
                'paramClass': Tea,
                'insertId': 'id'
            },
            'delete': {
                'sql': 'DELETE FROM tea WHERE id = :id'
            },
            'deleteAll': {
                'sql': 'DELETE FROM tea' 
            },
            'update': {
                'sql': 'UPDATE tea SET name = :name, cost = :cost, mod = :mod WHERE id = :id',
                'paramClass': Tea
            },
            'drop': {
                'sql': 'DROP TABLE tea'  
            },
            'create': {
                'sql': (
                        'CREATE TABLE IF NOT EXISTS tea ('
                            'id integer primary key autoincrement, '
                            'name text, '
                            'cost real, '
                            'mod date default current_date '
                        ');')
            }
        }
        Sql.__init__(self, db, queries)

if __name__ == '__main__':
    import sqlite3
    from datetime import datetime
    try:
        db = sqlite3.connect(database='test.db')
        
        # Build basic objects
        t1 = Tea(name='Earl Grey', cost=1.25)
        t2 = Tea(name='Lady Grey', cost=1.25)
        t3 = Tea(name='English Breakfast Tea', cost=1.35)
        t4 = Tea(name='Green Tea', cost=0.90)
        t5 = Tea(name='Black Tea', cost=0.50)
        t6 = Tea(name='White Tea', cost=2.25)
        t7 = Tea(name='Lemon Lift', cost=1.00)
        t8 = Tea(name='Mint Tea', cost=1.99)
        t9 = Tea(name='Dutch Express', cost=2.00)
        t10 = Tea(name='French Twist', cost=1.75)

        print (t1, t2, t3, t4, t5, t6, t7, t8, t9, t10)

        # Create the Dao
        dao = TeaSql(db)

        # Create the table
        dao.update('create')
        print 'Created'
        print

        # Insert Objects
        dao.update('insert', t1)
        print t1.id
        dao.update('insert', t2)
        print t2.id
        dao.update('insert', t3)
        print t3.id
        dao.update('insert', t4)
        print t4.id
        dao.update('insert', t5)
        print t5.id
        dao.update('insert', t6)
        print t6.id

        print 'Batch inserted %d elements' % dao.batch('insert', (t7, t8, t9, t10))
        print 'Ids not set automatically! (%s, %s, %s, %s)' % (t7.id, t8.id, t9.id, t10.id)
            
        # Print list
        print
        for obj in dao.queryForList('getAll'):
            print '(%s, %s, %s, %s)' % (obj.id, obj.name, obj.cost, obj.mod)
        print

        # Raise Prices
        print
        for obj in dao.queryForList('getAll'):
            obj.cost += (obj.cost * 0.1)
            obj.mod = datetime.now()
            dao.update('update', obj)

        # Print list
        print
        for obj in dao.queryForList('getAll'):
            print '(%s, %s, %s, %s)' % (obj.id, obj.name, obj.cost, obj.mod)
        print

        # Check Map
        print
        for key,objs in dao.queryForMap('getAll', 'id').iteritems():
            for obj in objs:
                print 'id %s -> (%s, %s, %s, %s)' % (key, obj.id, obj.name, obj.cost, obj.mod)
        print

        # Count Types of Tee
        print
        print 'Found %d number of teas' % dao.queryForObject('count')
        print 'Total %0.2f price of teas' % dao.queryForObject('sum')
        print 'Average %0.2f price of teas' % dao.queryForObject('avg')
        print

        # Delete Objects
        dao.update('deleteAll')
        print 'Deleted'
        
        # Drop table
        dao.update('drop')
        print 'Dropped'
        
    except sqlite3.Error,e:
        print
        print 'Uncaught Exception: ',str(e)
        print
    finally:
        db.close()

